unit SetupApiDevicesList;

// Fills a TStrings descendent with a list of all disk devices

interface

uses
  Windows, Classes, SysUtils;

procedure GetDisks(aStrings: TStrings);


implementation

uses
  Registry,SetupApi;


type
	TModuleHandle = HINST;

const
  INVALID_MODULEHANDLE_VALUE = TModuleHandle(0);   // Value designating an unassigned TModuleHandle or a failed loading


  function CheckOS(var VersionInfo: TOSVersionInfo): Integer;
begin
  VersionInfo.dwOSVersionInfoSize:= SizeOf(VersionInfo);
  if GetVersionEx(VersionInfo) then CheckOS:= VersionInfo.dwPlatformId
  else CheckOS:= -1;
end;

//------------------------------------------------------------------------------
procedure GetPortListFromReg(aStrings: TStrings);
var
  Reg: TRegistry;
  VersionInfo: TOSVersionInfo;
  List: TStringlist;

  procedure ScanRegHardware;
  var
    i: integer;
    PortName: string;
    LName: TStringList;
  begin
    if Reg.OpenKeyReadOnly('\hardware\devicemap\serialcomm') then
    begin
      LName:= TStringList.Create;
      Reg.GetValueNames(LName);
      for i:= 0 to LName.Count - 1 do
      begin
        if Reg.GetDataType(LName.Strings[i]) = rdString then
        begin
          PortName:= Reg.ReadString(LName.Strings[i]);
          if List.IndexOf(PortName) < 0 then
            List.Add(PortName);
        end
      end;
      LName.Free;
    end
  end;

  procedure ScanRegEnum(Key: String);
  var
    LKey: TStringList;
    LName: TStringList;
    i: Integer;
    Driver, PortName: String;
    PortSubClass: Byte;
  begin
    if not Reg.OpenKeyReadOnly(Key) then
      Exit;
    LName:= TStringList.Create;
    Reg.GetValueNames(LName);
    i:= LName.IndexOf('class');
    if i >= 0 then
    begin
      if (Reg.GetDataType('class') = rdString)
      and (LowerCase(Reg.ReadString('class')) = 'ports')
      and (Reg.GetDataType('driver') = rdString)
      and (Reg.GetDataType('portname') = rdString) then
      begin
        Driver:= Reg.ReadString('driver');
        PortName:= Reg.ReadString('portname');
        if Reg.OpenKeyReadOnly('\System\CurrentControlSet\Services\Class\'+ Driver)
        and (Reg.ReadBinaryData('PortSubClass',PortSubClass,1) = 1)
        and (PortSubClass = 1)
        and (List.IndexOf(PortName) < 0) then
          List.Add(PortName);
      end
    end;
    LName.Free;
    Reg.OpenKeyReadOnly(Key);
    if Reg.HasSubKeys then
    begin
      LKey:= TStringList.Create;
      Reg.GetKeyNames(LKey);
      for i:= 0 to LKey.Count - 1 do
        ScanRegEnum(Key + '\' + LKey[i]);
      LKey.Free;
    end;
  end;

begin
  Reg  := nil;
  List := TStringlist.Create;
  try
    if (CheckOS(VersionInfo) > 0)
    and (VersionInfo.dwMajorVersion >= 4) then
    begin
      Reg:= TRegistry.Create;
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      if VersionInfo.dwMajorVersion > 4 then
        ScanRegHardware //Win 2k, XP
      else if VersionInfo.dwMinorVersion > 0 then
        ScanRegEnum('\enum') //Win9x, mE
      else ScanRegHardware; //WinNT
      List.Sort;
      aStrings.Assign(List);
    end;
  finally
    List.Free;
    Reg.Free;
  end;
end;

function GetModuleSymbolEx(Module: TModuleHandle; SymbolName: string; var Accu: Boolean): Pointer;
begin
  Result := nil;
  if Module <> INVALID_MODULEHANDLE_VALUE then
	Result := GetProcAddress(Module, PChar(SymbolName));
  Accu := Accu and (Result <> nil);
end;


//------------------------------------------------------------------------------
// GET LIST USING SETUPAPI with friendly names.
//------------------------------------------------------------------------------
const
  DIGCF_DEFAULT         = $00000001;
  // only valid with DIGCF_DEVICEINTERFACE
  DIGCF_PRESENT         = $00000002;
  DIGCF_ALLCLASSES      = $00000004;
  DIGCF_PROFILE         = $00000008;
  DIGCF_DEVICEINTERFACE = $00000010;
  SPDRP_FRIENDLYNAME = $0000000C;

  // WinXP only:
  //GUID_DEVINTERFACE_COMPORT: TGUID = '{86e0d1e0-8089-11d0-9ce4-08003e301f73}';
  GUID_DEVINTERFACE_DISK: TGUID = '{53F56307-B6BF-11D0-94F2-00A0C91EFB8B}';

  // Win9x only:
  cComPortsGuid: TGUID = '{4D36E978-E325-11CE-BFC1-08002BE10318}';
  cComPortClassGuidTxt = '{4D36E978-E325-11CE-BFC1-08002BE10318}';

type
  // Device information structure (references a device instance
  // that is a member of a device information set)

  PSP_DEVINFO_DATA = ^SP_DEVINFO_DATA;

  TSetupDiGetCustomDevicePropertyA = function(DeviceInfoSet: HDEVINFO;
                                     DeviceInfoData: PSP_DEVINFO_DATA;
                                     CustomPropertyName: PChar;
                                     Flags: DWORD;
                                     PropertyRegDataType: PDWORD;
                                     PropertyBuffer: PChar;
                                     PropertyBufferSize: DWORD;
                                     RequiredSize: PDWORD): UINT; stdcall;
//

//------------------------------------------------------------------------------
procedure GetListFromSetupApi(aStrings: TStrings);
var
  iDev: Integer;
  RegDataType: Cardinal;
  reqSize:DWORD;
  prop:Cardinal;
  pszData:PByte;
  hinfo:   HDEVINFO;
  bResult: BOOL;
  devinfo: SP_DEVINFO_DATA;
  dwRequiredSize,dwPropertyRegDataType,dwAllocSz:Cardinal;
begin
  LoadSetupApi;
  if not Assigned(SetupDiGetClassDevs) then
	Exit;

  hinfo := SetupDiGetClassDevs(@GUID_DEVINTERFACE_DISK, nil, HWND(nil),
							   DIGCF_DEVICEINTERFACE or DIGCF_PRESENT or DIGCF_PROFILE);

  devinfo.ClassGuid.D1 := 0;
  devinfo.ClassGuid.D2 := 0;
  devinfo.ClassGuid.D3 := 0;
  devinfo.cbSize := SizeOf(SP_DEVINFO_DATA);

  iDev := 0;
   while SetupDiEnumDeviceInfo(hinfo, iDev, devinfo) do
	begin

	dwRequiredSize := 0;

	prop := SPDRP_PHYSICAL_DEVICE_OBJECT_NAME;
	// results on my computer:
	// \Device\Ide\IAAStorageDevice-1
	// \Device\Ide\IAAStorageDevice-2
	// \Device\0000008a                 (this one is a usb disk, use SPDRP_ENUMERATOR_NAME, returns USBSTOR)

//	 prop := SPDRP_ENUMERATOR_NAME; // results: IDE, USBSTOR, or other bus type.

//	 prop := SPDRP_LOCATION_INFORMATION; // a number like 1,2,3.


	{ SPDRP_DRIVER - driver guid }
	{ Get Size of property }
	 SetupDiGetDeviceRegistryProperty
				(hinfo,
				devinfo,
				prop,
				dwPropertyRegDataType,
				nil,
				0,
				dwRequiredSize);   { dwRequiredSize should be around 88 after this point, in unicode delphi }

	 if dwRequiredSize>0 then begin

		dwAllocSz := dwRequiredSize+4;
		pszData := AllocMem(dwAllocSz);
		bResult := SetupDiGetDeviceRegistryProperty
				(hinfo,
				devinfo,
				prop,
				dwPropertyRegDataType,
				pszData,
				dwAllocSz,
				dwRequiredSize);

		aStrings.Add(IntToStr(aStrings.Count)+': '+PChar(pszData));
		FreeMem(pszData);

	end;
	inc(iDev);
  end;
  SetupDiDestroyDeviceInfoList(hinfo);
end;


procedure GetDisks(aStrings: TStrings);
var
  iReg,iApi: integer;
  RegList,ApiList: TStringList;
begin
  ApiList := nil;
  RegList := TStringList.Create;
  try
	ApiList := TStringList.Create;
	//GetListFromReg(RegList);
	GetListFromSetupApi(ApiList);
	aStrings.Assign(ApiList);
  finally
    ApiList.Free;
    RegList.Free;
  end;
end;

//------------------------------------------------------------------------------

end.

